package com.dougmccune.steganography
{
	import flash.utils.ByteArray;
	
	public class SteganographyDecoder
	{
		//http://www.dreamincode.net/forums/showtopic27950.htm
		
		public var carrierBookmark:int;
		public var payloadBookmark:int;
		
		private var carrierBytes:ByteArray;
		private var payloadBytes:ByteArray;
		
	   	public function decodeBytes(carrierBytes:ByteArray):* {
	   		this.carrierBytes = carrierBytes;
	   		
	   		payloadBytes = new ByteArray();
	   		payloadBookmark = 0;
	   		
	   		var i:int;
	   		
	   		//first four encoded bytes are for the content type
	   		for(i=0; i<4; i++) {
	   			readPayloadByte();
	   		}
	   		
	   		//next four bytes are for the length of the message
	   		for(i=0; i<4; i++) {
	   			readPayloadByte();
	   		}
	   		
	   		payloadBytes.position = 0;
	   		
	   		var contentType:int = payloadBytes.readInt();
	   		var length:int = payloadBytes.readInt();
	   		
	   		//now read all the rest of the encoded bytes, these are either a compressed UTF string
	   		//or they are a zip archive containing files
	   		for(i=0; i<length; i++) {
	   			readPayloadByte();
	   		}
	   		
	   		var messageBytes:ByteArray = new ByteArray();
	   		payloadBytes.readBytes(messageBytes);
	   		
	   		if(contentType == SteganograpyContents.STRING) {
	   			//if we stored a String then we stored it using the compress() method, 
	   			//so now we do the opposite before reading the UTF string
	   			messageBytes.uncompress();
	   			return messageBytes.readUTF();
	   		}
	   		else if(contentType == SteganograpyContents.FILES) {
	   			return messageBytes;
	   		}
	   		else {
	   			return null;
	   		}
	   	}
	   	
	   	public function readPayloadByte():void
		{
			for(var i:int=0; i<8; i++) {
				payloadBytes[payloadBookmark] = ((payloadBytes[payloadBookmark] << 1) | (carrierBytes[carrierBookmark] & 1));
				carrierBookmark++;
			}
			
			payloadBookmark++;
		}
		
	}
}